import networkx as nx
import matplotlib.pyplot as plt
 

class DFS():
	#utility fucntion used by DFS which does recursive depth first search 
	def __DFSUtil(self, G, v, visited, sl): 
		visited[v] = True
		sl.append(v) 
		for i in G[v]:
			if not visited[i]:
				self.__DFSUtil(G, i, visited, sl)
		return sl
	


	#DFS traversal 
	def __dfs(self, G, source): 
		visited = [False]*(len(G.nodes()))
		sl = []		#a list that stores dfs forest starting with source node
		stack = [] #A nested list that stores all the DFS Forest's
		stack.append(self.__DFSUtil(G, source, visited, sl))
		for i in range(len(G.nodes())):
			if not visited[i]:
				sl = []
				stack.append(self.__DFSUtil(G, i, visited, sl))
		return stack
				


	#takes input from the file and creates a weighted graph
	def __CreateGraph(self, filename):
		G = nx.DiGraph()
		f = open(filename)
		n = int(f.readline())
		wtMatrix = []
		for i in range(n):
			list1 = map(int,(f.readline()).split())
			wtMatrix.append(list(list1))
		source = int(f.readline()) #source vertex from where DFS has to start
		#Adds egdes along with their weights to the graph 
		for i in range(n):
			for j in range(n):
				if wtMatrix[i][j] > 0:
						G.add_edge(i, j, length = wtMatrix[i][j]) 
		return G,source



	#marks all edges traversed through DFS with red
	def __DrawDFSPath(self, G, stack):
		pos = nx.spring_layout(G)
		options = {
		"node_color": "#A0CBE2",
		"edge_color": "#000000",
		"width": 3,
		"edge_cmap": plt.cm.Blues,
		"with_labels" : True,
		}
		nx.draw(G, pos, **options)  #with_labels=true is to show the node number in the output graph
		edge_labels = dict([((u,v,), d['length']) for u, v, d in G.edges(data = True)])
		nx.draw_networkx_edge_labels(G, pos, edge_labels = edge_labels, label_pos = 0.3, font_size = 11) #prints weight on all the edges
		for i in stack:
			#if there is more than one node in the dfs-forest, then print the corresponding edges
			if len(i) > 1:
				for j in i[ :(len(i)-1)]:
					if i[i.index(j)+1] in G[j]:
						nx.draw_networkx_edges(G, pos, edgelist = [(j,i[i.index(j)+1])], width = 3.5, alpha = 0.6, edge_color = 'r')
					else:
						#if in case the path was reversed because all the possible neighbours were visited, we need to find the adj node to it.
						for k in i[1::-1]: 
							if k in G[j]:
								nx.draw_networkx_edges(G, pos, edgelist = [(j,k)], width = 3.5, alpha = 0.6, edge_color = 'r')
								break
	
	#draws the graph and displays the weights on the edges
	def __DrawGraph(self, G):
		pos = nx.spring_layout(G)
		options = {
		"node_color": "#A0CBE2",
		"edge_color": "#000000",
		"width": 3,
		"edge_cmap": plt.cm.Blues,
		"with_labels" : True,
		}
		nx.draw(G, pos, **options)  #with_labels=true is to show the node number in the output graph
		edge_labels = dict([((u,v,), d['length']) for u, v, d in G.edges(data = True)])
		nx.draw_networkx_edge_labels(G, pos, edge_labels = edge_labels, label_pos = 0.3, font_size = 11) #prints weight on all the edges
		return pos

	def draw_graph(self, filename):
		G,source = self.__CreateGraph(filename)
		self.__DrawGraph(G)
		plt.show()							

	
	def depth_first_search(self, filename):
		G, source = self.__CreateGraph(filename)
		stack = self.__dfs(G, source)
		self.__DrawDFSPath(G, stack)
		plt.show()








import networkx as nx
import matplotlib.pyplot as plt
 
class Top_Sort:
    
    # A recursive function used by topologicalSort
    def __topologicalSortUtil(self, v, visited, stack, G):
 
        # Mark the current node as visited.
        visited[v] = True
 
        # Recur for all the vertices adjacent to this vertex
        for i in range(len(G.nodes())):
            if visited[i] == False:
                self.__topologicalSortUtil(i, visited, stack, G)
 
        # Push current vertex to stack which stores result
        stack.append(v)
 
    # The function to do Topological Sort. It uses recursive
    # topologicalSortUtil()
    def __topologicalSort(self, G):
        # Mark all the vertices as not visited
        visited = [False]*len(G.nodes())
        stack = []
 
        # Call the recursive helper function to store Topological
        # Sort starting from all vertices one by one
        for i in range(len(G.nodes())):
            if visited[i] == False:
                self.__topologicalSortUtil(i, visited, stack, G)
 
        # Print contents of the stack
        return stack[::-1]  # return list in reverse order
    
    def __CreateGraph(self, filename):
        G = nx.DiGraph()
        f = open(filename)
        n = int(f.readline())
        for i in range(n):
            adj_list = list(map(int, (f.readline()).split()))
            G.add_edge(adj_list[0], adj_list[1]) 
        return G

    #takes input from the file and creates a directed graph
    def __CreateResultGraph(self, sorted_list):
        D = nx.DiGraph()
        for i in range(len(sorted_list)-1): 
            D.add_edge(sorted_list[i], sorted_list[i+1]) 
        pos = nx.spring_layout(D)
        val_map = {}
        val_map[sorted_list[0]] = 'green'
        val_map[sorted_list[len(sorted_list)-1]] = 'red'
        values = [val_map.get(node, 'blue') for node in D.nodes()]
        options = {
		"node_color": values,
		"edge_color": "#000000",
		"width": 3,
		"edge_cmap": plt.cm.Blues,
		"with_labels" : True,
		}
        nx.draw(D, pos, **options)
    
    #draws the graph
    def __DrawGraph(self, G):
        pos = nx.spring_layout(G)
        options = {
		"node_color": "#A0CBE2",
		"edge_color": "#000000",
		"width": 3,
		"edge_cmap": plt.cm.Blues,
		"with_labels" : True,
		}
        nx.draw(G, pos, **options)  #with_labels=true is to show the node number in the output graph
        return pos

    def topological_sort(self, filename):
        G = self.__CreateGraph(filename=filename)
        plt.figure("Input Graph")
        pos = self.__DrawGraph(G)
        plt.figure("Graph after Topological Sort")
        sorted_list = self.__topologicalSort(G=G)
        self.__CreateResultGraph(sorted_list)
        plt.show()




import networkx as nx
import matplotlib.pyplot as plt
 

class BFS():

	#BFS traversal 
	def __bfs(self, G, source, pos): 
		visited = [False]*(len(G.nodes()))
		level = [False]*(len(G.nodes()))
		queue = []		#a queue for BFS traversal
		queue.append(source)
		visited[source] = True
		level[source] = 0
		while queue:
			curr_node = queue.pop(0)
			for i in G[curr_node]:  #iterates through all the possible vertices adjacent to the curr_node
				if not visited[i]:
					queue.append(i)
					visited[i] = True
					level[i] = level[curr_node] + 1
					nx.draw_networkx_edges(G, pos, edgelist = [(curr_node,i)], width = 3.5, alpha = 0.6, edge_color = 'r')
		
		return



	#takes input from the file and creates a weighted graph
	def __CreateGraph(self, filename):
		G = nx.DiGraph()
		f = open(filename)
		n = int(f.readline())
		wtMatrix = []
		for i in range(n):
			list1 = map(int, (f.readline()).split())
			wtMatrix.append(list(list1))
		
		source = int(f.readline()) #source vertex from where BFS has to start
		#Adds egdes along with their weights to the graph 
		for i in range(n):
			for j in range(n):	
				if wtMatrix[i][j] > 0:
					G.add_edge(i, j, length = wtMatrix[i][j]) 
		return G, source



	#draws the graph and displays the weights on the edges
	def __DrawGraph(self, G):
		pos = nx.spring_layout(G)
		options = {
		"node_color": "#A0CBE2",
		"edge_color": "#000000",
		"width": 3,
		"edge_cmap": plt.cm.Blues,
		"with_labels" : True,
		}
		nx.draw(G, pos, **options)  #with_labels=true is to show the node number in the output graph
		edge_labels = dict([((u,v,), d['length']) for u, v, d in G.edges(data = True)])
		nx.draw_networkx_edge_labels(G, pos, edge_labels = edge_labels, label_pos = 0.3, font_size = 11) #prints weight on all the edges
		return pos


	def draw_graph(self, filename):
		G,source = self.__CreateGraph(filename)
		self.__DrawGraph(G)
		plt.show()	

	
	def breadth_first_search(self, filename):
		G,source = self.__CreateGraph(filename)
		pos = self.__DrawGraph(G)
		self.__bfs(G, source, pos)
		plt.show()





from setuptools import _install_setup_requires, setup

with open("README.md", "r") as f:
    long_description=f.read()


setup(name='graph_algo_vis',
      classifiers = [
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent"
      ],
      version='0.2',
      extras_require = {
          "dev": ["networkx", "matplotlib",],
      },
      install_requires=[           
          'networkx',
          'matplotlib',
      ],
      keywords = ['Graph', 'Algorithms', 'BFS', 'DFS', 'Topological Sort'],
      description='Visualize Graph Algorithms',
      packages=['graph_algo_vis'],
      author="Akrash Sharma",
      author_email="akarshsharma654@gmail.com",
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="http://akrashsharma.netlify.app/",
      project_urls={
        "Graph Algorithms Visualization": "https://github.com/Akarsh654/Graph-Algorithms-Package",
      },
      python_requires=">=3.6",
      zip_safe=False)